<?php

declare(strict_types=1);

/**
 * This file is part of the MultiFlexi package
 *
 * https://github.com/VitexSoftware/php-vitexsoftware-rbczpremiumapi
 *
 * (c) Vítězslav Dvořák <http://vitexsoftware.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * Production.
 *
 * ##### API Overview - Accounts list and balance - Transaction overview (also for saving accounts) - Payments import - Statement list and download - FX rates  ##### Authentication Before making a call to Premium API, you need to register your app at our _Developer portal_. This is where you get the **ClientID** that your application must send in the request as `X-IBM-Client-Id`. This is the key that grants your app access to the API.  However, this may not be enough. Your application needs to use mTLS to call most operations here. Thus, you not only need _https_ but also a client certificate issued by us. The exception is two operations for FX rates that are accessible also without a client certificate.  Each bank client/user can issue several certificates. Each certificate can permit different sets of operations (http methods) on different bank accounts. All this must be configured in Internet Banking first by each bank client/user (bank clients need to look under _Settings_ and do not forget to download the certificate at the last step). The certificate is downloaded in **PKCS#12** format as **\\*.p12** file and protected by a password chosen by the bank client/user. Yes, your app needs the password as well to get use of the **\\*p12** file for establishing mTLS connection to the bank.  Client certificates issued in Internet Banking for bank clients/users have limited validity (e.g. **5 years**). However, **each year** certificates are automatically blocked and bank client/user must unblock them in Internet Banking. It is possible to do it in advance and prolong the time before the certificate is blocked. Your app should be prepared for these scenarios and it should communicate such cases to your user in advance to provide seamless service and high user-experience of your app.  ##### Rate Limiting The number of requests in each API operation is limited to 10 per client per sliding second and 5000 per client per sliding day. The exception is the 'Download Statement' operation with the limits lowered to 5 per client per sliding second and 1500 per client per sliding day. This is because it transports potentially sizeable binary files. The consumer must be able to handle HTTP status 429 in case of exceeding these limits.  Response headers `X-RateLimit-Limit-Second` and `X-RateLimit-Limit-Day` show the actual limits configured for the specific operation. Response headers `X-RateLimit-Remaining-Second` and `X-RateLimit-Remaining-Day` are returned to help prevent the limits from being exceeded.  ##### Notes Be aware, that in certain error situations, API can return specific error structures along with 5xx status code, which is not explicitely defined below.  ##### Quick Start Client Feel free to download a <a href=\"assets/PremiumApiClient.java\" download>simple Java client</a> that gives you quick access to our API.
 *
 * The version of the OpenAPI document: 1.1.20240910
 * Contact: info@vitexsoftware.cz
 * Generated by: https://openapi-generator.tech
 * Generator version: 7.13.0
 */

/**
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

namespace VitexSoftware\Raiffeisenbank\Model;

use VitexSoftware\Raiffeisenbank\ObjectSerializer;

/**
 * GetBatchDetail200ResponseBatchItemsInner Class Doc Comment.
 *
 * @category Class
 *
 * @author   OpenAPI Generator team
 *
 * @see     https://openapi-generator.tech
 *
 * @implements \ArrayAccess<string, mixed>
 */
class GetBatchDetail200ResponseBatchItemsInner implements \ArrayAccess, \JsonSerializable, ModelInterface
{
    public const DISCRIMINATOR = null;
    public const BATCH_TYPE_DPO = 'DPO';
    public const BATCH_TYPE_DD = 'DD';
    public const BATCH_TYPE_FPO = 'FPO';
    public const BATCH_TYPE_SEPA = 'SEPA';
    public const STATUS_DRAFT = 'DRAFT';
    public const STATUS_ERROR = 'ERROR';
    public const STATUS_FOR_SIGN = 'FOR_SIGN';
    public const STATUS_VERIFIED = 'VERIFIED';
    public const STATUS_PASSING_TO_BANK = 'PASSING_TO_BANK';
    public const STATUS_PASSED = 'PASSED';
    public const STATUS_PASSED_TO_BANK_WITH_ERROR = 'PASSED_TO_BANK_WITH_ERROR';
    public const STATUS_UNDISCLOSED = 'UNDISCLOSED';

    /**
     * The original name of the model.
     */
    protected static string $openAPIModelName = 'getBatchDetail_200_response_batchItems_inner';

    /**
     * Array of property to type mappings. Used for (de)serialization.
     *
     * @var string[]
     */
    protected static array $openAPITypes = [
        'accountInfo' => '\VitexSoftware\Raiffeisenbank\Model\GetBatchDetail200ResponseBatchItemsInnerAccountInfo',
        'numberOfPayments' => 'int',
        'sumAmount' => 'float',
        'sumAmountCurrencyId' => 'string',
        'batchType' => 'string',
        'status' => 'string',
        'assignedUserName' => 'string',
        'lastChangeDateTime' => '\DateTime',
    ];

    /**
     * Array of property to format mappings. Used for (de)serialization.
     *
     * @var string[]
     *
     * @phpstan-var array<string, string|null>
     *
     * @psalm-var array<string, string|null>
     */
    protected static array $openAPIFormats = [
        'accountInfo' => null,
        'numberOfPayments' => null,
        'sumAmount' => 'decimal',
        'sumAmountCurrencyId' => null,
        'batchType' => null,
        'status' => null,
        'assignedUserName' => null,
        'lastChangeDateTime' => 'date-time',
    ];

    /**
     * Array of nullable properties. Used for (de)serialization.
     *
     * @var bool[]
     */
    protected static array $openAPINullables = [
        'accountInfo' => false,
        'numberOfPayments' => false,
        'sumAmount' => false,
        'sumAmountCurrencyId' => false,
        'batchType' => false,
        'status' => false,
        'assignedUserName' => false,
        'lastChangeDateTime' => false,
    ];

    /**
     * If a nullable field gets set to null, insert it here.
     *
     * @var bool[]
     */
    protected array $openAPINullablesSetToNull = [];

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name.
     *
     * @var string[]
     */
    protected static array $attributeMap = [
        'accountInfo' => 'accountInfo',
        'numberOfPayments' => 'numberOfPayments',
        'sumAmount' => 'sumAmount',
        'sumAmountCurrencyId' => 'sumAmountCurrencyId',
        'batchType' => 'batchType',
        'status' => 'status',
        'assignedUserName' => 'assignedUserName',
        'lastChangeDateTime' => 'lastChangeDateTime',
    ];

    /**
     * Array of attributes to setter functions (for deserialization of responses).
     *
     * @var string[]
     */
    protected static array $setters = [
        'accountInfo' => 'setAccountInfo',
        'numberOfPayments' => 'setNumberOfPayments',
        'sumAmount' => 'setSumAmount',
        'sumAmountCurrencyId' => 'setSumAmountCurrencyId',
        'batchType' => 'setBatchType',
        'status' => 'setStatus',
        'assignedUserName' => 'setAssignedUserName',
        'lastChangeDateTime' => 'setLastChangeDateTime',
    ];

    /**
     * Array of attributes to getter functions (for serialization of requests).
     *
     * @var string[]
     */
    protected static array $getters = [
        'accountInfo' => 'getAccountInfo',
        'numberOfPayments' => 'getNumberOfPayments',
        'sumAmount' => 'getSumAmount',
        'sumAmountCurrencyId' => 'getSumAmountCurrencyId',
        'batchType' => 'getBatchType',
        'status' => 'getStatus',
        'assignedUserName' => 'getAssignedUserName',
        'lastChangeDateTime' => 'getLastChangeDateTime',
    ];

    /**
     * Associative array for storing property values.
     *
     * @var mixed[]
     */
    protected array $container = [];

    /**
     * Constructor.
     *
     * @param null|mixed[] $data Associated array of property values
     *                           initializing the model
     */
    public function __construct(?array $data = null)
    {
        $this->setIfExists('accountInfo', $data ?? [], null);
        $this->setIfExists('numberOfPayments', $data ?? [], null);
        $this->setIfExists('sumAmount', $data ?? [], null);
        $this->setIfExists('sumAmountCurrencyId', $data ?? [], null);
        $this->setIfExists('batchType', $data ?? [], null);
        $this->setIfExists('status', $data ?? [], null);
        $this->setIfExists('assignedUserName', $data ?? [], null);
        $this->setIfExists('lastChangeDateTime', $data ?? [], null);
    }

    /**
     * Gets the string presentation of the object.
     *
     * @return string
     */
    public function __toString()
    {
        return json_encode(
            ObjectSerializer::sanitizeForSerialization($this),
            \JSON_PRETTY_PRINT,
        );
    }

    /**
     * Array of property to type mappings. Used for (de)serialization.
     *
     * @return array
     */
    public static function openAPITypes()
    {
        return self::$openAPITypes;
    }

    /**
     * Array of property to format mappings. Used for (de)serialization.
     *
     * @return array
     */
    public static function openAPIFormats()
    {
        return self::$openAPIFormats;
    }

    /**
     * Checks if a property is nullable.
     */
    public static function isNullable(string $property): bool
    {
        return self::openAPINullables()[$property] ?? false;
    }

    /**
     * Checks if a nullable property is set to null.
     */
    public function isNullableSetToNull(string $property): bool
    {
        return \in_array($property, $this->getOpenAPINullablesSetToNull(), true);
    }

    /**
     * Array of attributes where the key is the local name,
     * and the value is the original name.
     *
     * @return array
     */
    public static function attributeMap()
    {
        return self::$attributeMap;
    }

    /**
     * Array of attributes to setter functions (for deserialization of responses).
     *
     * @return array
     */
    public static function setters()
    {
        return self::$setters;
    }

    /**
     * Array of attributes to getter functions (for serialization of requests).
     *
     * @return array
     */
    public static function getters()
    {
        return self::$getters;
    }

    /**
     * The original name of the model.
     *
     * @return string
     */
    public function getModelName()
    {
        return self::$openAPIModelName;
    }

    /**
     * Gets allowable values of the enum.
     *
     * @return string[]
     */
    public function getBatchTypeAllowableValues()
    {
        return [
            self::BATCH_TYPE_DPO,
            self::BATCH_TYPE_DD,
            self::BATCH_TYPE_FPO,
            self::BATCH_TYPE_SEPA,
        ];
    }

    /**
     * Gets allowable values of the enum.
     *
     * @return string[]
     */
    public function getStatusAllowableValues()
    {
        return [
            self::STATUS_DRAFT,
            self::STATUS_ERROR,
            self::STATUS_FOR_SIGN,
            self::STATUS_VERIFIED,
            self::STATUS_PASSING_TO_BANK,
            self::STATUS_PASSED,
            self::STATUS_PASSED_TO_BANK_WITH_ERROR,
            self::STATUS_UNDISCLOSED,
        ];
    }

    /**
     * Show all the invalid properties with reasons.
     *
     * @return array invalid properties with reasons
     */
    public function listInvalidProperties()
    {
        $invalidProperties = [];

        if (null !== $this->container['sumAmountCurrencyId'] && (mb_strlen((string) $this->container['sumAmountCurrencyId']) > 3)) {
            $invalidProperties[] = "invalid value for 'sumAmountCurrencyId', the character length must be smaller than or equal to 3.";
        }

        if (null !== $this->container['sumAmountCurrencyId'] && (mb_strlen((string) $this->container['sumAmountCurrencyId']) < 3)) {
            $invalidProperties[] = "invalid value for 'sumAmountCurrencyId', the character length must be bigger than or equal to 3.";
        }

        $allowedValues = $this->getBatchTypeAllowableValues();

        if (null !== $this->container['batchType'] && !\in_array($this->container['batchType'], $allowedValues, true)) {
            $invalidProperties[] = sprintf(
                "invalid value '%s' for 'batchType', must be one of '%s'",
                $this->container['batchType'],
                implode("', '", $allowedValues),
            );
        }

        $allowedValues = $this->getStatusAllowableValues();

        if (null !== $this->container['status'] && !\in_array($this->container['status'], $allowedValues, true)) {
            $invalidProperties[] = sprintf(
                "invalid value '%s' for 'status', must be one of '%s'",
                $this->container['status'],
                implode("', '", $allowedValues),
            );
        }

        return $invalidProperties;
    }

    /**
     * Validate all the properties in the model
     * return true if all passed.
     *
     * @return bool True if all properties are valid
     */
    public function valid()
    {
        return \count($this->listInvalidProperties()) === 0;
    }

    /**
     * Gets accountInfo.
     *
     * @return null|\VitexSoftware\Raiffeisenbank\Model\GetBatchDetail200ResponseBatchItemsInnerAccountInfo
     */
    public function getAccountInfo()
    {
        return $this->container['accountInfo'];
    }

    /**
     * Sets accountInfo.
     *
     * @param null|\VitexSoftware\Raiffeisenbank\Model\GetBatchDetail200ResponseBatchItemsInnerAccountInfo $accountInfo accountInfo
     *
     * @return self
     */
    public function setAccountInfo($accountInfo)
    {
        if (null === $accountInfo) {
            throw new \InvalidArgumentException('non-nullable accountInfo cannot be null');
        }

        $this->container['accountInfo'] = $accountInfo;

        return $this;
    }

    /**
     * Gets numberOfPayments.
     *
     * @return null|int
     */
    public function getNumberOfPayments()
    {
        return $this->container['numberOfPayments'];
    }

    /**
     * Sets numberOfPayments.
     *
     * @param null|int $numberOfPayments Number of payments within the batch
     *
     * @return self
     */
    public function setNumberOfPayments($numberOfPayments)
    {
        if (null === $numberOfPayments) {
            throw new \InvalidArgumentException('non-nullable numberOfPayments cannot be null');
        }

        $this->container['numberOfPayments'] = $numberOfPayments;

        return $this;
    }

    /**
     * Gets sumAmount.
     *
     * @return null|float
     */
    public function getSumAmount()
    {
        return $this->container['sumAmount'];
    }

    /**
     * Sets sumAmount.
     *
     * @param null|float $sumAmount Sum amount
     *
     * @return self
     */
    public function setSumAmount($sumAmount)
    {
        if (null === $sumAmount) {
            throw new \InvalidArgumentException('non-nullable sumAmount cannot be null');
        }

        $this->container['sumAmount'] = $sumAmount;

        return $this;
    }

    /**
     * Gets sumAmountCurrencyId.
     *
     * @return null|string
     */
    public function getSumAmountCurrencyId()
    {
        return $this->container['sumAmountCurrencyId'];
    }

    /**
     * Sets sumAmountCurrencyId.
     *
     * @param null|string $sumAmountCurrencyId The currency folder identification (CATALOG: CURRENCIES)
     *
     * @return self
     */
    public function setSumAmountCurrencyId($sumAmountCurrencyId)
    {
        if (null === $sumAmountCurrencyId) {
            throw new \InvalidArgumentException('non-nullable sumAmountCurrencyId cannot be null');
        }

        if (mb_strlen((string) $sumAmountCurrencyId) > 3) {
            throw new \InvalidArgumentException('invalid length for $sumAmountCurrencyId when calling GetBatchDetail200ResponseBatchItemsInner., must be smaller than or equal to 3.');
        }

        if (mb_strlen((string) $sumAmountCurrencyId) < 3) {
            throw new \InvalidArgumentException('invalid length for $sumAmountCurrencyId when calling GetBatchDetail200ResponseBatchItemsInner., must be bigger than or equal to 3.');
        }

        $this->container['sumAmountCurrencyId'] = $sumAmountCurrencyId;

        return $this;
    }

    /**
     * Gets batchType.
     *
     * @return null|string
     */
    public function getBatchType()
    {
        return $this->container['batchType'];
    }

    /**
     * Sets batchType.
     *
     * @param null|string $batchType Batch transaction package payment type
     *
     * @return self
     */
    public function setBatchType($batchType)
    {
        if (null === $batchType) {
            throw new \InvalidArgumentException('non-nullable batchType cannot be null');
        }

        $allowedValues = $this->getBatchTypeAllowableValues();

        if (!\in_array($batchType, $allowedValues, true)) {
            throw new \InvalidArgumentException(
                sprintf(
                    "Invalid value '%s' for 'batchType', must be one of '%s'",
                    $batchType,
                    implode("', '", $allowedValues),
                ),
            );
        }

        $this->container['batchType'] = $batchType;

        return $this;
    }

    /**
     * Gets status.
     *
     * @return null|string
     */
    public function getStatus()
    {
        return $this->container['status'];
    }

    /**
     * Sets status.
     *
     * @param null|string $status Bacth transaction package status
     *
     * @return self
     */
    public function setStatus($status)
    {
        if (null === $status) {
            throw new \InvalidArgumentException('non-nullable status cannot be null');
        }

        $allowedValues = $this->getStatusAllowableValues();

        if (!\in_array($status, $allowedValues, true)) {
            throw new \InvalidArgumentException(
                sprintf(
                    "Invalid value '%s' for 'status', must be one of '%s'",
                    $status,
                    implode("', '", $allowedValues),
                ),
            );
        }

        $this->container['status'] = $status;

        return $this;
    }

    /**
     * Gets assignedUserName.
     *
     * @return null|string
     */
    public function getAssignedUserName()
    {
        return $this->container['assignedUserName'];
    }

    /**
     * Sets assignedUserName.
     *
     * @param null|string $assignedUserName Name of user assigned to batch transaction package
     *
     * @return self
     */
    public function setAssignedUserName($assignedUserName)
    {
        if (null === $assignedUserName) {
            throw new \InvalidArgumentException('non-nullable assignedUserName cannot be null');
        }

        $this->container['assignedUserName'] = $assignedUserName;

        return $this;
    }

    /**
     * Gets lastChangeDateTime.
     *
     * @return null|\DateTime
     */
    public function getLastChangeDateTime()
    {
        return $this->container['lastChangeDateTime'];
    }

    /**
     * Sets lastChangeDateTime.
     *
     * @param null|\DateTime $lastChangeDateTime Date and time of last change of batch transaction package
     *
     * @return self
     */
    public function setLastChangeDateTime($lastChangeDateTime)
    {
        if (null === $lastChangeDateTime) {
            throw new \InvalidArgumentException('non-nullable lastChangeDateTime cannot be null');
        }

        $this->container['lastChangeDateTime'] = $lastChangeDateTime;

        return $this;
    }
    /**
     * Returns true if offset exists. False otherwise.
     *
     * @param int $offset Offset
     */
    public function offsetExists($offset): bool
    {
        return isset($this->container[$offset]);
    }

    /**
     * Gets offset.
     *
     * @param int $offset Offset
     *
     * @return null|mixed
     */
    #[\ReturnTypeWillChange]
    public function offsetGet($offset)
    {
        return $this->container[$offset] ?? null;
    }

    /**
     * Sets value based on offset.
     *
     * @param null|int $offset Offset
     * @param mixed    $value  Value to be set
     */
    public function offsetSet($offset, $value): void
    {
        if (null === $offset) {
            $this->container[] = $value;
        } else {
            $this->container[$offset] = $value;
        }
    }

    /**
     * Unsets offset.
     *
     * @param int $offset Offset
     */
    public function offsetUnset($offset): void
    {
        unset($this->container[$offset]);
    }

    /**
     * Serializes the object to a value that can be serialized natively by json_encode().
     *
     * @see https://www.php.net/manual/en/jsonserializable.jsonserialize.php
     *
     * @return mixed returns data which can be serialized by json_encode(), which is a value
     *               of any type other than a resource
     */
    #[\ReturnTypeWillChange]
    public function jsonSerialize()
    {
        return ObjectSerializer::sanitizeForSerialization($this);
    }

    /**
     * Gets a header-safe presentation of the object.
     *
     * @return string
     */
    public function toHeaderValue()
    {
        return json_encode(ObjectSerializer::sanitizeForSerialization($this));
    }

    /**
     * Array of nullable properties.
     */
    protected static function openAPINullables(): array
    {
        return self::$openAPINullables;
    }

    /**
     * Array of nullable field names deliberately set to null.
     *
     * @return bool[]
     */
    private function getOpenAPINullablesSetToNull(): array
    {
        return $this->openAPINullablesSetToNull;
    }

    /**
     * Setter - Array of nullable field names deliberately set to null.
     *
     * @param bool[] $openAPINullablesSetToNull
     */
    private function setOpenAPINullablesSetToNull(array $openAPINullablesSetToNull): void
    {
        $this->openAPINullablesSetToNull = $openAPINullablesSetToNull;
    }

    /**
     * Sets $this->container[$variableName] to the given data or to the given default Value; if $variableName
     * is nullable and its value is set to null in the $fields array, then mark it as "set to null" in the
     * $this->openAPINullablesSetToNull array.
     *
     * @param mixed $defaultValue
     */
    private function setIfExists(string $variableName, array $fields, $defaultValue): void
    {
        if (self::isNullable($variableName) && \array_key_exists($variableName, $fields) && null === $fields[$variableName]) {
            $this->openAPINullablesSetToNull[] = $variableName;
        }

        $this->container[$variableName] = $fields[$variableName] ?? $defaultValue;
    }
}
