<?php

declare(strict_types=1);

/**
 * This file is part of the MultiFlexi package
 *
 * https://github.com/VitexSoftware/php-vitexsoftware-rbczpremiumapi
 *
 * (c) Vítězslav Dvořák <http://vitexsoftware.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * Production.
 *
 * ##### API Overview - Accounts list and balance - Transaction overview (also for saving accounts) - Payments import - Statement list and download - FX rates  ##### Authentication Before making a call to Premium API, you need to register your app at our _Developer portal_. This is where you get the **ClientID** that your application must send in the request as `X-IBM-Client-Id`. This is the key that grants your app access to the API.  However, this may not be enough. Your application needs to use mTLS to call most operations here. Thus, you not only need _https_ but also a client certificate issued by us. The exception is two operations for FX rates that are accessible also without a client certificate.  Each bank client/user can issue several certificates. Each certificate can permit different sets of operations (http methods) on different bank accounts. All this must be configured in Internet Banking first by each bank client/user (bank clients need to look under _Settings_ and do not forget to download the certificate at the last step). The certificate is downloaded in **PKCS#12** format as **\\*.p12** file and protected by a password chosen by the bank client/user. Yes, your app needs the password as well to get use of the **\\*p12** file for establishing mTLS connection to the bank.  Client certificates issued in Internet Banking for bank clients/users have limited validity (e.g. **5 years**). However, **each year** certificates are automatically blocked and bank client/user must unblock them in Internet Banking. It is possible to do it in advance and prolong the time before the certificate is blocked. Your app should be prepared for these scenarios and it should communicate such cases to your user in advance to provide seamless service and high user-experience of your app.  ##### Rate Limiting The number of requests in each API operation is limited to 10 per client per sliding second and 5000 per client per sliding day. The exception is the 'Download Statement' operation with the limits lowered to 5 per client per sliding second and 1500 per client per sliding day. This is because it transports potentially sizeable binary files. The consumer must be able to handle HTTP status 429 in case of exceeding these limits.  Response headers `X-RateLimit-Limit-Second` and `X-RateLimit-Limit-Day` show the actual limits configured for the specific operation. Response headers `X-RateLimit-Remaining-Second` and `X-RateLimit-Remaining-Day` are returned to help prevent the limits from being exceeded.  ##### Notes Be aware, that in certain error situations, API can return specific error structures along with 5xx status code, which is not explicitely defined below.  ##### Quick Start Client Feel free to download a <a href=\"assets/PremiumApiClient.java\" download>simple Java client</a> that gives you quick access to our API.
 *
 * The version of the OpenAPI document: 1.1.20240910
 * Contact: info@vitexsoftware.cz
 * Generated by: https://openapi-generator.tech
 * Generator version: 7.13.0
 */

/**
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

namespace VitexSoftware\Raiffeisenbank\PremiumAPI;

use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\MultipartStream;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\RequestOptions;
use VitexSoftware\Raiffeisenbank\ApiClient;
use VitexSoftware\Raiffeisenbank\ApiException;
use VitexSoftware\Raiffeisenbank\Configuration;
use VitexSoftware\Raiffeisenbank\HeaderSelector;
use VitexSoftware\Raiffeisenbank\ObjectSerializer;

/**
 * GetTransactionListApi Class Doc Comment.
 *
 * @category Class
 *
 * @author   OpenAPI Generator team
 *
 * @see     https://openapi-generator.tech
 */
class GetTransactionListApi
{
    /**
     * @var string[] *
     */
    public const contentTypes = [
        'getTransactionList' => [
            'application/json',
        ],
    ];

    protected ClientInterface $client;

    protected Configuration $config;

    protected HeaderSelector $headerSelector;

    /**
     * @var int Host index
     */
    protected int $hostIndex = 0;

    /**
     * ClientID obtained from Developer Portal - when you registered your app with us.
     */
    protected string $xIBMClientId = '';

    /**
     * Use the /rbcz/premium/mock/* path for endpoints ?
     */
    protected bool $mockMode = false;

    /**
     * the end IP address of the client application (no server) in IPv4 or IPv6 format. If the bank client (your user) uses a browser by which he accesses your server app, we need to know the IP address of his browser. Always provide the closest IP address to the real end-user possible. (optional).
     *
     * @var string Description
     */
    protected string $SUIPAddress = '';

    /**
     * @param int $hostIndex (Optional) host index to select the list of hosts if defined in the OpenAPI spec
     */
    public function __construct(
        ?ClientInterface $client = null,
        ?Configuration $config = null,
        ?HeaderSelector $selector = null,
        int $hostIndex = 0,
    ) {
        $this->client = $client ?: new ApiClient();
        $this->config = $config ?: Configuration::getDefaultConfiguration();
        $this->headerSelector = $selector ?: new HeaderSelector();
        $this->hostIndex = $hostIndex;

        if (method_exists($this->client, 'getXIBMClientId')) {
            $this->setXIBMClientId($this->client->getXIBMClientId());
        }

        if (method_exists($this->client, 'getpSUIPAddress')) {
            $this->setSUIPAddress($this->client->getpSUIPAddress());
        }

        if (method_exists($this->client, 'getMockMode')) {
            $this->setMockMode($this->client->getMockMode());
        }
    }

    /**
     * Keep ClientID obtained from Developer Portal.
     *
     * @param string $clientId Description
     */
    public function setXIBMClientId($clientId): self
    {
        $this->xIBMClientId = $clientId;

        return $this;
    }

    /**
     * Give you ClientID obtained from Developer Portal.
     */
    public function getXIBMClientId(): string
    {
        return $this->xIBMClientId;
    }

    /**
     * @param string $SUIPAddress IP address of a client
     */
    public function setSUIPAddress($SUIPAddress): self
    {
        $this->SUIPAddress;

        return $this;
    }

    /**
     * @param bool $mocking Use mocking api for development purposes ?
     */
    public function setMockMode($mocking): self
    {
        $this->mockMode = $mocking;

        return $this;
    }

    /**
     * Set the host index.
     *
     * @param int $hostIndex Host index (required)
     */
    public function setHostIndex($hostIndex): self
    {
        $this->hostIndex = $hostIndex;

        return $this;
    }

    /**
     * Get the host index.
     *
     * @return int Host index
     */
    public function getHostIndex()
    {
        return $this->hostIndex;
    }

    /**
     * @return Configuration
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Operation getTransactionList.
     *
     * @param string    $xRequestId    Unique request id provided by consumer application for reference and auditing. (required)
     * @param string    $accountNumber Account number for which to get list of transactions in national format without 0 padding. (required)
     * @param string    $currencyCode  Currency code of the account in ISO-4217 standard (e.g. czk, eur, usd) (required)
     * @param \DateTime $from          Defines date (and optionally time) from which transactions will be requested. If no time is specified then 00:00:00.0 (Central European  Time) will be used. Example values - 2021-08-01 or 2021-08-01T10:00:00.0Z (required)
     * @param \DateTime $to            Defines date (and optionally time) until which transactions will be requested. If no time is specified then 23:59:59.999 (Central European  Time) will be used. Example values - 2021-08-02 or 2021-08-02T14:00:00.0Z (required)
     * @param null|int  $page          Page number to be requested. The first page is 1. (optional)
     * @param string    $contentType   The value for the Content-Type header. Check self::contentTypes['getTransactionList'] to see the possible values for this operation
     *
     * @throws \InvalidArgumentException
     * @throws \VitexSoftware\Raiffeisenbank\ApiException on non-2xx response or if the response body is not in the expected format
     *
     * @return \VitexSoftware\Raiffeisenbank\Model\GetBalance401Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance403Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance404Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance429Response|\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response|\VitexSoftware\Raiffeisenbank\Model\GetTransactionList400Response
     */
    public function getTransactionList($xRequestId, $accountNumber, $currencyCode, $from, $to, $page = null, string $contentType = self::contentTypes['getTransactionList'][0])
    {
        [$response] = $this->getTransactionListWithHttpInfo($xRequestId, $accountNumber, $currencyCode, $from, $to, $page, $contentType);

        return $response;
    }

    /**
     * Operation getTransactionListWithHttpInfo.
     *
     * @param string    $xRequestId    Unique request id provided by consumer application for reference and auditing. (required)
     * @param string    $accountNumber Account number for which to get list of transactions in national format without 0 padding. (required)
     * @param string    $currencyCode  Currency code of the account in ISO-4217 standard (e.g. czk, eur, usd) (required)
     * @param \DateTime $from          Defines date (and optionally time) from which transactions will be requested. If no time is specified then 00:00:00.0 (Central European  Time) will be used. Example values - 2021-08-01 or 2021-08-01T10:00:00.0Z (required)
     * @param \DateTime $to            Defines date (and optionally time) until which transactions will be requested. If no time is specified then 23:59:59.999 (Central European  Time) will be used. Example values - 2021-08-02 or 2021-08-02T14:00:00.0Z (required)
     * @param null|int  $page          Page number to be requested. The first page is 1. (optional)
     * @param string    $contentType   The value for the Content-Type header. Check self::contentTypes['getTransactionList'] to see the possible values for this operation
     *
     * @throws \InvalidArgumentException
     * @throws \VitexSoftware\Raiffeisenbank\ApiException on non-2xx response or if the response body is not in the expected format
     *
     * @return array of \VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response|\VitexSoftware\Raiffeisenbank\Model\GetTransactionList400Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance401Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance403Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance404Response|\VitexSoftware\Raiffeisenbank\Model\GetBalance429Response, HTTP status code, HTTP response headers (array of strings)
     */
    public function getTransactionListWithHttpInfo($xRequestId, $accountNumber, $currencyCode, $from, $to, $page = null, string $contentType = self::contentTypes['getTransactionList'][0])
    {
        $request = $this->getTransactionListRequest($xRequestId, $accountNumber, $currencyCode, $from, $to, $page, $contentType);

        try {
            $options = $this->createHttpClientOption();

            try {
                $response = $this->client->send($request, $options);
            } catch (RequestException $e) {
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    (int) $e->getCode(),
                    $e->getResponse() ? $e->getResponse()->getHeaders() : null,
                    $e->getResponse() ? (string) $e->getResponse()->getBody() : null,
                );
            } catch (ConnectException $e) {
                throw new ApiException(
                    "[{$e->getCode()}] {$e->getMessage()}",
                    (int) $e->getCode(),
                    null,
                    null,
                );
            }

            $statusCode = $response->getStatusCode();

            switch ($statusCode) {
                case 200:
                    if ('\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response' === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ('\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response' !== 'string') {
                            try {
                                $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                            } catch (\JsonException $exception) {
                                throw new ApiException(
                                    sprintf(
                                        'Error JSON decoding server response (%s)',
                                        $request->getUri(),
                                    ),
                                    $statusCode,
                                    $response->getHeaders(),
                                    $content,
                                );
                            }
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, '\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response', []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
                case 400:
                    if ('\VitexSoftware\Raiffeisenbank\Model\GetTransactionList400Response' === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ('\VitexSoftware\Raiffeisenbank\Model\GetTransactionList400Response' !== 'string') {
                            try {
                                $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                            } catch (\JsonException $exception) {
                                throw new ApiException(
                                    sprintf(
                                        'Error JSON decoding server response (%s)',
                                        $request->getUri(),
                                    ),
                                    $statusCode,
                                    $response->getHeaders(),
                                    $content,
                                );
                            }
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, '\VitexSoftware\Raiffeisenbank\Model\GetTransactionList400Response', []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
                case 401:
                    if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance401Response' === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance401Response' !== 'string') {
                            try {
                                $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                            } catch (\JsonException $exception) {
                                throw new ApiException(
                                    sprintf(
                                        'Error JSON decoding server response (%s)',
                                        $request->getUri(),
                                    ),
                                    $statusCode,
                                    $response->getHeaders(),
                                    $content,
                                );
                            }
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, '\VitexSoftware\Raiffeisenbank\Model\GetBalance401Response', []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
                case 403:
                    if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance403Response' === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance403Response' !== 'string') {
                            try {
                                $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                            } catch (\JsonException $exception) {
                                throw new ApiException(
                                    sprintf(
                                        'Error JSON decoding server response (%s)',
                                        $request->getUri(),
                                    ),
                                    $statusCode,
                                    $response->getHeaders(),
                                    $content,
                                );
                            }
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, '\VitexSoftware\Raiffeisenbank\Model\GetBalance403Response', []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
                case 404:
                    if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance404Response' === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance404Response' !== 'string') {
                            try {
                                $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                            } catch (\JsonException $exception) {
                                throw new ApiException(
                                    sprintf(
                                        'Error JSON decoding server response (%s)',
                                        $request->getUri(),
                                    ),
                                    $statusCode,
                                    $response->getHeaders(),
                                    $content,
                                );
                            }
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, '\VitexSoftware\Raiffeisenbank\Model\GetBalance404Response', []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
                case 429:
                    if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance429Response' === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ('\VitexSoftware\Raiffeisenbank\Model\GetBalance429Response' !== 'string') {
                            try {
                                $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                            } catch (\JsonException $exception) {
                                throw new ApiException(
                                    sprintf(
                                        'Error JSON decoding server response (%s)',
                                        $request->getUri(),
                                    ),
                                    $statusCode,
                                    $response->getHeaders(),
                                    $content,
                                );
                            }
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, '\VitexSoftware\Raiffeisenbank\Model\GetBalance429Response', []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
            }

            if ($statusCode < 200 || $statusCode > 299) {
                throw new ApiException(
                    sprintf(
                        '[%d] Error connecting to the API (%s)',
                        $statusCode,
                        (string) $request->getUri(),
                    ),
                    $statusCode,
                    $response->getHeaders(),
                    (string) $response->getBody(),
                );
            }

            $returnType = '\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response';

            if ($returnType === '\SplFileObject') {
                $content = $response->getBody(); // stream goes to serializer
            } else {
                $content = (string) $response->getBody();

                if ($returnType !== 'string') {
                    if ($statusCode === 204) {
                        $content = [];
                    } else {
                        try {
                            $content = json_decode($content, false, 512, \JSON_THROW_ON_ERROR);
                        } catch (\JsonException $exception) {
                            throw new ApiException(
                                sprintf(
                                    'Error JSON decoding server response (%s)',
                                    $request->getUri(),
                                ),
                                $statusCode,
                                $response->getHeaders(),
                                $content,
                            );
                        }
                    }
                }
            }

            return [
                ObjectSerializer::deserialize($content, $returnType, []),
                $response->getStatusCode(),
                $response->getHeaders(),
            ];
        } catch (ApiException $e) {
            switch ($e->getCode()) {
                case 200:
                    $data = ObjectSerializer::deserialize(
                        $e->getResponseBody(),
                        '\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response',
                        $e->getResponseHeaders(),
                    );
                    $e->setResponseObject($data);

                    break;
                case 400:
                    $data = ObjectSerializer::deserialize(
                        $e->getResponseBody(),
                        '\VitexSoftware\Raiffeisenbank\Model\GetTransactionList400Response',
                        $e->getResponseHeaders(),
                    );
                    $e->setResponseObject($data);

                    break;
                case 401:
                    $data = ObjectSerializer::deserialize(
                        $e->getResponseBody(),
                        '\VitexSoftware\Raiffeisenbank\Model\GetBalance401Response',
                        $e->getResponseHeaders(),
                    );
                    $e->setResponseObject($data);

                    break;
                case 403:
                    $data = ObjectSerializer::deserialize(
                        $e->getResponseBody(),
                        '\VitexSoftware\Raiffeisenbank\Model\GetBalance403Response',
                        $e->getResponseHeaders(),
                    );
                    $e->setResponseObject($data);

                    break;
                case 404:
                    $data = ObjectSerializer::deserialize(
                        $e->getResponseBody(),
                        '\VitexSoftware\Raiffeisenbank\Model\GetBalance404Response',
                        $e->getResponseHeaders(),
                    );
                    $e->setResponseObject($data);

                    break;
                case 429:
                    $data = ObjectSerializer::deserialize(
                        $e->getResponseBody(),
                        '\VitexSoftware\Raiffeisenbank\Model\GetBalance429Response',
                        $e->getResponseHeaders(),
                    );
                    $e->setResponseObject($data);

                    break;
            }

            throw $e;
        }
    }

    /**
     * Operation getTransactionListAsync.
     *
     * @param string    $xRequestId    Unique request id provided by consumer application for reference and auditing. (required)
     * @param string    $accountNumber Account number for which to get list of transactions in national format without 0 padding. (required)
     * @param string    $currencyCode  Currency code of the account in ISO-4217 standard (e.g. czk, eur, usd) (required)
     * @param \DateTime $from          Defines date (and optionally time) from which transactions will be requested. If no time is specified then 00:00:00.0 (Central European  Time) will be used. Example values - 2021-08-01 or 2021-08-01T10:00:00.0Z (required)
     * @param \DateTime $to            Defines date (and optionally time) until which transactions will be requested. If no time is specified then 23:59:59.999 (Central European  Time) will be used. Example values - 2021-08-02 or 2021-08-02T14:00:00.0Z (required)
     * @param null|int  $page          Page number to be requested. The first page is 1. (optional)
     * @param string    $contentType   The value for the Content-Type header. Check self::contentTypes['getTransactionList'] to see the possible values for this operation
     *
     * @throws \InvalidArgumentException
     *
     * @return \GuzzleHttp\Promise\PromiseInterface
     */
    public function getTransactionListAsync($xRequestId, $accountNumber, $currencyCode, $from, $to, $page = null, string $contentType = self::contentTypes['getTransactionList'][0])
    {
        return $this->getTransactionListAsyncWithHttpInfo($xRequestId, $accountNumber, $currencyCode, $from, $to, $page, $contentType)
            ->then(
                static function ($response) {
                    return $response[0];
                },
            );
    }

    /**
     * Operation getTransactionListAsyncWithHttpInfo.
     *
     * @param string    $xRequestId    Unique request id provided by consumer application for reference and auditing. (required)
     * @param string    $accountNumber Account number for which to get list of transactions in national format without 0 padding. (required)
     * @param string    $currencyCode  Currency code of the account in ISO-4217 standard (e.g. czk, eur, usd) (required)
     * @param \DateTime $from          Defines date (and optionally time) from which transactions will be requested. If no time is specified then 00:00:00.0 (Central European  Time) will be used. Example values - 2021-08-01 or 2021-08-01T10:00:00.0Z (required)
     * @param \DateTime $to            Defines date (and optionally time) until which transactions will be requested. If no time is specified then 23:59:59.999 (Central European  Time) will be used. Example values - 2021-08-02 or 2021-08-02T14:00:00.0Z (required)
     * @param null|int  $page          Page number to be requested. The first page is 1. (optional)
     * @param string    $contentType   The value for the Content-Type header. Check self::contentTypes['getTransactionList'] to see the possible values for this operation
     *
     * @throws \InvalidArgumentException
     *
     * @return \GuzzleHttp\Promise\PromiseInterface
     */
    public function getTransactionListAsyncWithHttpInfo($xRequestId, $accountNumber, $currencyCode, $from, $to, $page = null, string $contentType = self::contentTypes['getTransactionList'][0])
    {
        $returnType = '\VitexSoftware\Raiffeisenbank\Model\GetTransactionList200Response';
        $request = $this->getTransactionListRequest($xRequestId, $accountNumber, $currencyCode, $from, $to, $page, $contentType);

        return $this->client
            ->sendAsync($request, $this->createHttpClientOption())
            ->then(
                static function ($response) use ($returnType) {
                    if ($returnType === '\SplFileObject') {
                        $content = $response->getBody(); // stream goes to serializer
                    } else {
                        $content = (string) $response->getBody();

                        if ($returnType !== 'string') {
                            $content = json_decode($content);
                        }
                    }

                    return [
                        ObjectSerializer::deserialize($content, $returnType, []),
                        $response->getStatusCode(),
                        $response->getHeaders(),
                    ];
                },
                static function ($exception): void {
                    $response = $exception->getResponse();
                    $statusCode = $response->getStatusCode();

                    throw new ApiException(
                        sprintf(
                            '[%d] Error connecting to the API (%s)',
                            $statusCode,
                            $exception->getRequest()->getUri(),
                        ),
                        $statusCode,
                        $response->getHeaders(),
                        (string) $response->getBody(),
                    );
                },
            );
    }

    /**
     * Create request for operation 'getTransactionList'.
     *
     * @param string    $xRequestId    Unique request id provided by consumer application for reference and auditing. (required)
     * @param string    $accountNumber Account number for which to get list of transactions in national format without 0 padding. (required)
     * @param string    $currencyCode  Currency code of the account in ISO-4217 standard (e.g. czk, eur, usd) (required)
     * @param \DateTime $from          Defines date (and optionally time) from which transactions will be requested. If no time is specified then 00:00:00.0 (Central European  Time) will be used. Example values - 2021-08-01 or 2021-08-01T10:00:00.0Z (required)
     * @param \DateTime $to            Defines date (and optionally time) until which transactions will be requested. If no time is specified then 23:59:59.999 (Central European  Time) will be used. Example values - 2021-08-02 or 2021-08-02T14:00:00.0Z (required)
     * @param null|int  $page          Page number to be requested. The first page is 1. (optional)
     * @param string    $contentType   The value for the Content-Type header. Check self::contentTypes['getTransactionList'] to see the possible values for this operation
     *
     * @throws \InvalidArgumentException
     *
     * @return \GuzzleHttp\Psr7\Request
     */
    public function getTransactionListRequest($xRequestId, $accountNumber, $currencyCode, $from, $to, $page = null, string $contentType = self::contentTypes['getTransactionList'][0])
    {
        $xIBMClientId = $this->getXIBMClientId();
        $pSUIPAddress = $this->SUIPAddress;

        // verify the required parameter 'xIBMClientId' is set
        if ($xIBMClientId === null || (\is_array($xIBMClientId) && \count($xIBMClientId) === 0)) {
            throw new \InvalidArgumentException(
                'Missing the required parameter $xIBMClientId when calling getTransactionList',
            );
        }

        // verify the required parameter 'xRequestId' is set
        if ($xRequestId === null || (\is_array($xRequestId) && \count($xRequestId) === 0)) {
            throw new \InvalidArgumentException(
                'Missing the required parameter $xRequestId when calling getTransactionList',
            );
        }

        if (\strlen((string) $xRequestId) > 60) {
            throw new \InvalidArgumentException('invalid length for "$xRequestId" when calling GetTransactionListApi.getTransactionList, must be smaller than or equal to 60.');
        }

        if (!preg_match('/[a-zA-Z0-9\\-_:]{1,60}/', (string) $xRequestId)) {
            throw new \InvalidArgumentException('invalid value for "xRequestId" when calling GetTransactionListApi.getTransactionList, must conform to the pattern /[a-zA-Z0-9\\-_:]{1,60}/.');
        }

        // verify the required parameter 'accountNumber' is set
        if ($accountNumber === null || (\is_array($accountNumber) && \count($accountNumber) === 0)) {
            throw new \InvalidArgumentException(
                'Missing the required parameter $accountNumber when calling getTransactionList',
            );
        }

        if (\strlen((string) $accountNumber) > 10) {
            throw new \InvalidArgumentException('invalid length for "$accountNumber" when calling GetTransactionListApi.getTransactionList, must be smaller than or equal to 10.');
        }

        if (!preg_match('/[0-9]{1,10}/', (string) $accountNumber)) {
            throw new \InvalidArgumentException('invalid value for "accountNumber" when calling GetTransactionListApi.getTransactionList, must conform to the pattern /[0-9]{1,10}/.');
        }

        // verify the required parameter 'currencyCode' is set
        if ($currencyCode === null || (\is_array($currencyCode) && \count($currencyCode) === 0)) {
            throw new \InvalidArgumentException(
                'Missing the required parameter $currencyCode when calling getTransactionList',
            );
        }

        if (\strlen((string) $currencyCode) > 3) {
            throw new \InvalidArgumentException('invalid length for "$currencyCode" when calling GetTransactionListApi.getTransactionList, must be smaller than or equal to 3.');
        }

        if (!preg_match('/[A-Z]{1,3}/', (string) $currencyCode)) {
            throw new \InvalidArgumentException('invalid value for "currencyCode" when calling GetTransactionListApi.getTransactionList, must conform to the pattern /[A-Z]{1,3}/.');
        }

        // verify the required parameter 'from' is set
        if ($from === null || (\is_array($from) && \count($from) === 0)) {
            throw new \InvalidArgumentException(
                'Missing the required parameter $from when calling getTransactionList',
            );
        }

        // verify the required parameter 'to' is set
        if ($to === null || (\is_array($to) && \count($to) === 0)) {
            throw new \InvalidArgumentException(
                'Missing the required parameter $to when calling getTransactionList',
            );
        }

        if ($pSUIPAddress !== null && \strlen((string) $pSUIPAddress) > 39) {
            throw new \InvalidArgumentException('invalid length for "$pSUIPAddress" when calling GetTransactionListApi.getTransactionList, must be smaller than or equal to 39.');
        }

        if ($page !== null && $page > 99999) {
            throw new \InvalidArgumentException('invalid value for "$page" when calling GetTransactionListApi.getTransactionList, must be smaller than or equal to 99999.');
        }

        if ($page !== null && $page < 1) {
            throw new \InvalidArgumentException('invalid value for "$page" when calling GetTransactionListApi.getTransactionList, must be bigger than or equal to 1.');
        }

        $resourcePath = '/rbcz/premium/api/accounts/{accountNumber}/{currencyCode}/transactions';

        if ($this->mockMode === true) {
            $resourcePath = str_replace('/rbcz/premium/api/', '/rbcz/premium/mock/', $resourcePath);
        }

        $formParams = [];
        $queryParams = [];
        $headerParams = [];
        $httpBody = '';
        $multipart = false;

        // query params
        $queryParams = array_merge($queryParams, ObjectSerializer::toQueryValue(
            $from,
            'from', // param base name
            'string', // openApiType
            '', // style
            false, // explode
            true, // required
        ) ?? []);
        // query params
        $queryParams = array_merge($queryParams, ObjectSerializer::toQueryValue(
            $to,
            'to', // param base name
            'string', // openApiType
            '', // style
            false, // explode
            true, // required
        ) ?? []);
        // query params
        $queryParams = array_merge($queryParams, ObjectSerializer::toQueryValue(
            $page,
            'page', // param base name
            'integer', // openApiType
            '', // style
            false, // explode
            false, // required
        ) ?? []);

        // header params
        if ($xIBMClientId !== null) {
            $headerParams['X-IBM-Client-Id'] = ObjectSerializer::toHeaderValue($xIBMClientId);
        }

        // header params
        if ($xRequestId !== null) {
            $headerParams['X-Request-Id'] = ObjectSerializer::toHeaderValue($xRequestId);
        }

        // header params
        if ($pSUIPAddress !== null) {
            $headerParams['PSU-IP-Address'] = ObjectSerializer::toHeaderValue($pSUIPAddress);
        }

        // path params
        if ($accountNumber !== null) {
            $resourcePath = str_replace(
                '{accountNumber}',
                ObjectSerializer::toPathValue($accountNumber),
                $resourcePath,
            );
        }

        // path params
        if ($currencyCode !== null) {
            $resourcePath = str_replace(
                '{currencyCode}',
                ObjectSerializer::toPathValue($currencyCode),
                $resourcePath,
            );
        }

        $headers = $this->headerSelector->selectHeaders(
            ['application/json'],
            $contentType,
            $multipart,
        );

        // for model (json/xml)
        if (\count($formParams) > 0) {
            if ($multipart) {
                $multipartContents = [];

                foreach ($formParams as $formParamName => $formParamValue) {
                    $formParamValueItems = \is_array($formParamValue) ? $formParamValue : [$formParamValue];

                    foreach ($formParamValueItems as $formParamValueItem) {
                        $multipartContents[] = [
                            'name' => $formParamName,
                            'contents' => $formParamValueItem,
                        ];
                    }
                }

                // for HTTP post (form)
                $httpBody = new MultipartStream($multipartContents);
            } elseif (str_contains(strtolower($headers['Content-Type']), strtolower('application/json'))) {
                // if Content-Type contains "application/json", json_encode the form parameters
                $httpBody = \GuzzleHttp\Utils::jsonEncode($formParams);
            } else {
                // for HTTP post (form)
                $httpBody = ObjectSerializer::buildQuery($formParams);
            }
        }

        $defaultHeaders = [];

        if ($this->config->getUserAgent()) {
            $defaultHeaders['User-Agent'] = $this->config->getUserAgent();
        }

        $headers = array_merge(
            $defaultHeaders,
            $headerParams,
            $headers,
        );

        $operationHost = $this->config->getHost();
        $query = ObjectSerializer::buildQuery($queryParams);

        return new Request(
            'GET',
            $operationHost.$resourcePath.($query ? "?{$query}" : ''),
            $headers,
            $httpBody,
        );
    }

    /**
     * Create http client option.
     *
     * @throws \RuntimeException on file opening failure
     *
     * @return array of http client options
     */
    protected function createHttpClientOption()
    {
        $options = [];

        if ($this->config->getDebug()) {
            $options[RequestOptions::DEBUG] = fopen($this->config->getDebugFile(), 'ab');

            if (!$options[RequestOptions::DEBUG]) {
                throw new \RuntimeException('Failed to open the debug file: '.$this->config->getDebugFile());
            }
        }

        return $options;
    }
}
